package rdbtoonto.converters;

import java.util.HashSet;
import java.util.Vector;

import com.hp.hpl.jena.ontology.OntClass;

import rdbtoonto.RDBToOnto;
import rdbtoonto.dbmodel.ForeignKey;
import rdbtoonto.dbmodel.TableDef;

/**
 * A sample converter implementation to show how new converters can be added <BR>
 * in RDBToOnto. <BR>
 * AroundATable produces an ontology from a source table. The only tables that<BR>
 * are considered as relevant are the source table and those which are directly<BR>
 * linked to it through foreign key relationships.<BR>
 * The source table is a user inputdefined as a global option.<BR> 
 * The way new types of local contraints can be added is also illustrated.<BR>
 * A new local constraint type is added to allow the inclusion of comments<BR>
 * on the classes derived from tables.<BR>
 * Specific guis are implemented for both the new global option and local<BR>
 * constraint.
 */

public class AroundOneTable extends RDBToOntoConverter {
	public static boolean _WithInstances;
	public static String _SourceTabName;
	
	public AroundOneTable() {
		super();
	}
	
	public static boolean initConvConf() {
		RDBToOntoConverter.initConvConf();
		_WithInstances = RDBToOnto.booleanInit(RDBToOnto._R2OConf,"conversion[@withInstances]",true);
		return true;
	}
	
	/*
	 * initNewProject: When creating a new project, the complete current project definition 
	 * should be reset. 
	 * initNewProject method from RDBToOntoConverter does most of the work. It just remain to reset
	 * attributes that are specific to the convertzer at hand.
	 * @see rdbtoonto.converters.RDBToOntoConverter#initNewProject()
	 */
	
	public static boolean initNewProject() {
		boolean res = RDBToOntoConverter.initNewProject();
		if (!res) {
			return false;
		}
		_SourceTabName = "-none-";
		return true;
	}
	
	
	/* initConvProject: called to initialize project conversion options from the project description 
	 * (defined in the configuration object "RDBToOnto._DbConf").
	 * The call to "RDBToOntoConverter.initCommonConvProject" initializes the common options.
	 * The rest of the implementation handles options that are specific to this new converter.
	 * A call to "readAllLCSs" should be included.
	 * @see rdbtoonto.converters.RDBToOntoConverter#initConvProject()
	 */
	
	public static boolean initConvProject() {	
		boolean res = RDBToOntoConverter.initCommonConvProject();
		if (!res) {
			return false;
		}	
		_WithInstances = RDBToOnto.booleanInit(RDBToOnto._PrjConf,"conversion[@withInstances]",_WithInstances);
		_SourceTabName =  RDBToOnto.initProp(RDBToOnto._PrjConf,"conversion.sourceTable", "-none-");
		readAllLCSs();
		return true;
	}
	
	/* 
	 *  Implementation of the transformation method
	 * @see rdbtoonto.converters.RDBToOntoConverter#convert()
	 */
	
	public boolean convert() {
		RDBToOnto.medPrintln(" Applying AroundOneTable Converter - Conversion to OWL ...");
		
		if (_SourceTabName.equals("-none-")) {
			System.err.println("RDBToOnto Error: AroundOneTable converter: no source table provided");
			return false;
		}
		
		TableDef srcTable = RDBToOnto.getDatabase().getDBSchema().getTableDef(_SourceTabName);
		
		identifyClassesFromSourceTable(srcTable);
		
		identifyClassToClassProperties();
		
		if (_WithInstances) {
			populateClasses();
		}
		return true;
	}
	
	public void identifyClassesFromSourceTable(TableDef srcTable) {	
		RDBToOnto.lowPrintln("Generating classes ...");
		HashSet<TableDef> concernedTables = findConcernedTables(srcTable);	
		RDBToOnto.medPrintln("Number of concerned tables:" + concernedTables.size());
		for(TableDef table : concernedTables) {
			if (!isAssociation(table)) {
				OntClass cls = createClassWithAttributes(table,null,null);
				setComment(table.getName(),cls);
			}
		}
	}
	
	public HashSet<TableDef> findConcernedTables(TableDef srcTable) {
		
		HashSet<TableDef> concernedTables = new HashSet<TableDef>();
		concernedTables.add(srcTable);
		
		Vector<ForeignKey> fkeys;
		for(TableDef td : RDBToOnto.getDatabase().getDBSchema().getTableDefs()) {
			fkeys = td.getForeignKeys();
			if (td == srcTable) { // add "foreign" tables of the source table
				for(ForeignKey fkey : fkeys) {
					concernedTables.add(fkey.getExternalTable());
				}
			}
			else { // add tables that have source table as foreign tables
				boolean srcFound = false;
				TableDef ftab;
				for(int i=0; i < fkeys.size() && !srcFound ; i++) {
					ftab = fkeys.elementAt(i).getExternalTable();
					if (srcTable == ftab) {
						srcFound = true;
						concernedTables.add(td);
					}
				}
			}
		}
		return concernedTables;
	}
	
	
	public void setComment(String tabName, OntClass cls ) {
		if (_WithLocalConstraints) {	
			LocalConstraintSet cd = _LocalConstraintSets.get(tabName);
			if ((cd != null) && (cd._Active)) { 
				if (cd instanceof AroundOneTableLCS) {
					String comment = ((AroundOneTableLCS)cd)._Comment;			
					if (!comment.equals("-none-")) {
						cls.addComment(comment, null);
					}
				}
			}
		}
	}

	/*
	 *  saveConvPartOfProject: upper level method to save global options and 
	 *  local constraints of the current project.
	 *  overrides saveConvPartOfProject from RDBToOntoConverter since new 
	 *  global options and local constraints are added.
	 *  Calls to saveCommonConvPart() and saveAllLCSs() are always needed.  
	 * @see rdbtoonto.converters.RDBToOntoConverter#saveConvPartOfProject()
	 */
	
	public static void saveConvPartOfProject() {
		saveCommonConvPart();
		RDBToOnto._PrjConf.addProperty("conversion.sourceTable",_SourceTabName);
		saveAllLCSs();
	}
	
	/*
	 *  readLCS: overrides a base method from RDBToOnto.
	 *  However, call to the base method is still needed to read the  
	 *  local constraints that are common to all converters.
	 *  
	*/
	
	public AroundOneTableLCS readLCS(String iter) {
		LocalConstraintSet cd = super.readLCS(iter);
		AroundOneTableLCS tcd = new AroundOneTableLCS(cd);
		String comm =  RDBToOnto._PrjConf.getString("conversion.lcs" + iter + ".comment");		
		if (comm == null) {
			comm = "-none-";
		}
		tcd._Comment = comm;
		return tcd;
	}
	
	/*
	 *  saveLCS: overrides a base method saveLCS from RDBToOnto.
	 *  However, call to the base method is still needed to save the  
	 *  local constraints that are common to all converters.
	 *  Note that instanceof testing and the way of recovering the object
	 *  are motivated by the fact that the constraint set might be of a 
	 *  different specific type if the converter has been dynamically 
	 *  changed during the session.  
	 * @see rdbtoonto.converters.RDBToOntoConverter#saveLCS(rdbtoonto.converters.LocalConstraintSet)
	 */
	
	public void saveLCS(LocalConstraintSet cd) {
		super.saveLCS(cd);
		AroundOneTableLCS aotcd; 
		if (cd instanceof AroundOneTableLCS) {
			aotcd = (AroundOneTableLCS)cd;	
		}
		else{
			aotcd = new AroundOneTableLCS(cd);
		}
		if (!aotcd._Comment.equals("-none-")) {
			RDBToOnto._PrjConf.addProperty("conversion.lcs.comment", aotcd._Comment);
		}
	}	
	
};